==> cicada-0.9.20.crate: cicada-0.9.20/src/calculator/mod.rs <==
use pest::Parser;
use pest::iterators::{Pair, Pairs};
use pest::prec_climber::*;

#[derive(Parser)]
#[grammar = "calculator/grammar.pest"]
struct Calculator;

lazy_static! {
    static ref PREC_CLIMBER: PrecClimber<Rule> = {
        use Rule::*;
        use Assoc::*;

        PrecClimber::new(vec![
            Operator::new(add, Left) | Operator::new(subtract, Left),
            Operator::new(multiply, Left) | Operator::new(divide, Left),
            Operator::new(power, Right)
        ])
    };
}

pub fn eval_int(expression: Pairs<Rule>) -> i64 {
    PREC_CLIMBER.climb(
        expression,
        |pair: Pair<Rule>| match pair.as_rule() {
            Rule::num => pair.as_str().parse::<i64>().unwrap(),
            Rule::expr => eval_int(pair.into_inner()),
            _ => unreachable!(),
        },
        |lhs: i64, op: Pair<Rule>, rhs: i64| match op.as_rule() {
            Rule::add      => lhs + rhs,
            Rule::subtract => lhs - rhs,
            Rule::multiply => lhs * rhs,
            Rule::divide   => {
                if rhs == 0 {
                    (lhs as f64 / 0.0) as i64
                } else {
                    lhs / rhs
                }
            }
            Rule::power    => lhs.pow(rhs as u32),
            _ => unreachable!(),
        },
    )
}

pub fn eval_float(expression: Pairs<Rule>) -> f64 {
    PREC_CLIMBER.climb(
        expression,
        |pair: Pair<Rule>| match pair.as_rule() {
            Rule::num => pair.as_str().parse::<f64>().unwrap(),
            Rule::expr => eval_float(pair.into_inner()),
            _ => unreachable!(),
        },
        |lhs: f64, op: Pair<Rule>, rhs: f64| match op.as_rule() {
            Rule::add      => lhs + rhs,
            Rule::subtract => lhs - rhs,
            Rule::multiply => lhs * rhs,
            Rule::divide   => lhs / rhs,
            Rule::power    => lhs.powf(rhs),
            _ => unreachable!(),
        },
    )
}

pub fn calculate(line: &str) -> Result<pest::iterators::Pairs<'_, Rule>, pest::error::Error<Rule>> {
    Calculator::parse(Rule::calculation, line)
}

==> cicada-0.9.23.crate: cicada-0.9.23/src/calculator/mod.rs <==
use std::num::Wrapping as W;

use pest::Parser;
use pest::iterators::{Pair, Pairs};
use pest::prec_climber::*;

#[derive(Parser)]
#[grammar = "calculator/grammar.pest"]
struct Calculator;

lazy_static! {
    static ref PREC_CLIMBER: PrecClimber<Rule> = {
        use Rule::*;
        use Assoc::*;

        PrecClimber::new(vec![
            Operator::new(add, Left) | Operator::new(subtract, Left),
            Operator::new(multiply, Left) | Operator::new(divide, Left),
            Operator::new(power, Right)
        ])
    };
}

pub fn eval_int(expression: Pairs<Rule>) -> i64 {
    PREC_CLIMBER.climb(
        expression,
        |pair: Pair<Rule>| match pair.as_rule() {
            Rule::num => {
                match pair.as_str().parse::<i64>() {
                    Ok(n) => n,
                    Err(_) => 0,
                }
            }
            Rule::expr => eval_int(pair.into_inner()),
            _ => unreachable!(),
        },
        |lhs: i64, op: Pair<Rule>, rhs: i64| match op.as_rule() {
            Rule::add      => (W(lhs) + W(rhs)).0,
            Rule::subtract => (W(lhs) - W(rhs)).0,
            Rule::multiply => (W(lhs) * W(rhs)).0,
            Rule::divide   => {
                if rhs == 0 {
                    (lhs as f64 / 0.0) as i64
                } else {
                    (W(lhs) / W(rhs)).0
                }
            }
            Rule::power    => lhs.pow(rhs as u32),
            _ => unreachable!(),
        },
    )
}

pub fn eval_float(expression: Pairs<Rule>) -> f64 {
    PREC_CLIMBER.climb(
        expression,
        |pair: Pair<Rule>| match pair.as_rule() {
            Rule::num => {
                match pair.as_str().parse::<f64>() {
                    Ok(f) => f,
                    Err(_) => 0.0,
                }
            }
            Rule::expr => eval_float(pair.into_inner()),
            _ => unreachable!(),
        },
        |lhs: f64, op: Pair<Rule>, rhs: f64| match op.as_rule() {
            Rule::add      => lhs + rhs,
            Rule::subtract => lhs - rhs,
            Rule::multiply => lhs * rhs,
            Rule::divide   => lhs / rhs,
            Rule::power    => lhs.powf(rhs),
            _ => unreachable!(),
        },
    )
}

pub fn calculate(line: &str) -> Result<pest::iterators::Pairs<'_, Rule>, pest::error::Error<Rule>> {
    Calculator::parse(Rule::calculation, line)
}
